@@ -20,14 +20,14 @@ import six
 import socket
 
 import dask
-from dask.base import tokenize, normalize_token, Base
+from dask.base import tokenize, normalize_token, Base, collections_to_dsk
 from dask.core import flatten, get_dependencies
 from dask.compatibility import apply, unicode
 from dask.context import _globals
 from toolz import first, groupby, merge, valmap, keymap
 from tornado import gen
 from tornado.gen import Return, TimeoutError
-from tornado.locks import Event
+from tornado.locks import Event, Condition
 from tornado.ioloop import IOLoop, PeriodicCallback
 from tornado.queues import Queue
 
@@ -37,7 +37,7 @@ from .compatibility import Queue as pyQueue, Empty, isqueue
 from .core import connect, rpc, clean_exception, CommClosedError
 from .protocol import to_serialize
 from .protocol.pickle import dumps, loads
-from .worker import dumps_function, dumps_task
+from .worker import dumps_task
 from .utils import (All, sync, funcname, ignoring, queue_to_iterator,
         tokey, log_errors, str_graph)
 from .versions import get_versions
@@ -246,6 +246,9 @@ class Future(WrappedKey):
 
     __repr__ = __str__
 
+    def __await__(self):
+        return self._result().__await__()
+
 
 class FutureState(object):
     """A Future's internal state.
@@ -382,6 +385,8 @@ class Client(object):
 
         from distributed.channels import ChannelClient
         ChannelClient(self)  # registers itself on construction
+        from distributed.recreate_exceptions import ReplayExceptionClient
+        ReplayExceptionClient(self)
 
     def __str__(self):
         if hasattr(self, '_loop_thread'):
@@ -390,7 +395,7 @@ class Client(object):
                     self.__class__.__name__,
                     self.scheduler.address, len(n), sum(n.values()))
         elif hasattr(self, 'scheduler'):
-            return '<%s: scheduler="%r">' % (
+            return '<%s: scheduler=%r>' % (
                     self.__class__.__name__, self.scheduler.address)
         else:
             return '<%s: not connected>' % (self.__class__.__name__,)
@@ -495,7 +500,7 @@ class Client(object):
         comm = yield connect(self.scheduler.address,
                              timeout=timeout)
 
-        ident = yield self.scheduler.identity()
+        yield self.scheduler.identity()
 
         yield comm.write({'op': 'register-client',
                           'client': self.id, 'reply': False})
@@ -590,7 +595,12 @@ class Client(object):
     def _handle_key_in_memory(self, key=None, type=None, workers=None):
         state = self.futures.get(key)
         if state is not None:
-            type = loads(type) if type and not state.type else None
+            if type and not state.type:  # Type exists and not yet set
+                type = loads(type)
+                # Here, `type` may be a str if actual type failed
+                # serializing in Worker
+            else:
+                type = None
             state.finish(type)
 
     def _handle_lost_data(self, key=None):
@@ -776,7 +786,6 @@ class Client(object):
             f = self.submit(func, *args, **kwargs)
             q_out.put(f)
 
-
     def map(self, func, *iterables, **kwargs):
         """ Map a function on a sequence of arguments
 
@@ -862,7 +871,7 @@ class Client(object):
                     " for a sequence of length %d" % (len(workers), len(keys)))
                 restrictions = dict(zip(keys, workers))
             else:
-                restrictions = {key: workers for key in keys}
+                restrictions = {k: workers for k in keys}
         elif workers is None:
             restrictions = {}
         else:
@@ -877,7 +886,7 @@ class Client(object):
         priority = dict(zip(keys, range(len(keys))))
 
         if resources:
-            resources = {key: resources for key in keys}
+            resources = {k: resources for k in keys}
         else:
             resources = None
 
@@ -885,7 +894,7 @@ class Client(object):
                 loose_restrictions, priority=priority, resources=resources)
         logger.debug("map(%s, ...)", funcname(func))
 
-        return [futures[tokey(key)] for key in keys]
+        return [futures[tokey(k)] for k in keys]
 
     @gen.coroutine
     def _gather(self, futures, errors='raise'):
@@ -913,18 +922,21 @@ class Client(object):
             bad_keys = set()
             for key in keys:
                 if (key not in self.futures or
-                    self.futures[key].status in failed):
+                        self.futures[key].status in failed):
                     exceptions.add(key)
                     if errors == 'raise':
                         try:
                             st = self.futures[key]
-                            six.reraise(type(st.exception),
-                                        st.exception,
-                                        st.traceback)
+                            exception = st.exception
+                            traceback = st.traceback
                         except (AttributeError, KeyError):
                             six.reraise(CancelledError,
                                         CancelledError(key),
                                         None)
+                        else:
+                            six.reraise(type(exception),
+                                        exception,
+                                        traceback)
                     if errors == 'skip':
                         bad_keys.add(key)
                         bad_data[key] = None
@@ -970,8 +982,8 @@ class Client(object):
         Accepts a future, nested container of futures, iterator, or queue.
         The return type will match the input type.
 
-        Parametrs
-        ---------
+        Parameters
+        ----------
         futures: Collection of futures
             This can be a possibly nested collection of Future objects.
             Collections can be lists, sets, iterators, queues or dictionaries
@@ -1156,6 +1168,7 @@ class Client(object):
         else:
             return sync(self.loop, self._scatter, data, workers=workers,
                         broadcast=broadcast)
+
     @gen.coroutine
     def _cancel(self, futures):
         keys = {tokey(f.key) for f in futures_of(futures)}
@@ -1188,7 +1201,7 @@ class Client(object):
                 coroutines.append(self.scheduler.publish_put(keys=keys,
                     name=tokey(name), data=dumps(data), client=self.id))
 
-            outs = yield coroutines
+            yield coroutines
 
     def publish_dataset(self, **kwargs):
         """
@@ -1750,7 +1763,7 @@ class Client(object):
                 assert os.path.exists(os.path.join(c, name[:-4]))
                 return c
 
-        responses = yield self._run(unzip, nanny=True)
+        yield self._run(unzip, nanny=True)
         raise gen.Return(name[:-4])
 
     def upload_environment(self, name, zipfile):
@@ -2373,25 +2386,8 @@ class Client(object):
         return restrictions, loose_restrictions
 
     @staticmethod
-    def collections_to_dsk(collections, optimize_graph=True, **kwargs):
-        """
-        Convert many collections into a single dask graph, after optimization
-        """
-        optimizations = _globals.get('optimizations', [])
-        if optimize_graph:
-            groups = groupby(lambda x: x._optimize, collections)
-            groups = {opt: [merge([v.dask for v in val]),
-                           [v._keys() for v in val]]
-                      for opt, val in groups.items()}
-            for opt in optimizations:
-                groups = {k: [opt(dsk, keys), keys]
-                          for k, (dsk, keys) in groups.items()}
-            dsk = merge([opt(dsk, keys, **kwargs)
-                         for opt, (dsk, keys) in groups.items()])
-        else:
-            dsk = merge(c.dask for c in collections)
-
-        return dsk
+    def collections_to_dsk(collections, *args, **kwargs):
+        return collections_to_dsk(collections, *args, **kwargs)
 
 
 Executor = Client
@@ -2530,6 +2526,7 @@ class AsCompleted(object):
         self.queue = pyQueue()
         self.lock = Lock()
         self.loop = loop or default_client().loop
+        self.condition = Condition()
 
         if futures:
             for future in futures:
@@ -2543,6 +2540,7 @@ class AsCompleted(object):
             if not self.futures[future]:
                 del self.futures[future]
             self.queue.put_nowait(future)
+            self.condition.notify()
 
     def add(self, future):
         """ Add a future to the collection
@@ -2558,12 +2556,23 @@ class AsCompleted(object):
     def __iter__(self):
         return self
 
+    def __aiter__(self):
+        return self
+
     def __next__(self):
         with self.lock:
             if not self.futures and self.queue.empty():
                 raise StopIteration()
         return self.queue.get()
 
+    @gen.coroutine
+    def __anext__(self):
+        if not self.futures and self.queue.empty():
+            raise StopAsyncIteration  # flake8: noqa
+        while self.queue.empty():
+            yield self.condition.wait()
+        raise gen.Return(self.queue.get())
+
     next = __next__
 
 
