@@ -14,9 +14,11 @@ from tornado.tcpserver import TCPServer
 
 from .. import config
 from ..compatibility import finalize
-from ..utils import ensure_bytes
-from .core import (connectors, listeners, Comm, Listener, CommClosedError,
-                   parse_host_port, unparse_host_port)
+from ..utils import ensure_bytes, ensure_ip, get_ip, get_ipv6
+
+from .registry import Backend, backends
+from .addressing import parse_host_port, unparse_host_port
+from .core import Comm, Connector, Listener, CommClosedError
 from .utils import (to_frames, from_frames,
                     get_tcp_server_address, ensure_concrete_host)
 
@@ -134,12 +136,10 @@ class TCP(Comm):
         return self._peer_addr
 
     @gen.coroutine
-    def read(self, deserialize=None):
+    def read(self):
         stream = self.stream
         if stream is None:
             raise CommClosedError
-        if deserialize is None:
-            deserialize = self.deserialize
 
         try:
             n_frames = yield stream.read_bytes(8)
@@ -158,7 +158,7 @@ class TCP(Comm):
             self.stream = None
             convert_stream_closed_error(e)
 
-        msg = from_frames(frames, deserialize=deserialize)
+        msg = from_frames(frames, deserialize=self.deserialize)
         raise gen.Return(msg)
 
     @gen.coroutine
@@ -296,5 +296,36 @@ class TCPListener(Listener):
         self.comm_handler(comm)
 
 
-connectors['tcp'] = TCPConnector()
-listeners['tcp'] = TCPListener
+class TCPBackend(Backend):
+
+    # I/O
+
+    def get_connector(self):
+        return TCPConnector()
+
+    def get_listener(self, loc, handle_comm, deserialize):
+        return TCPListener(loc, handle_comm, deserialize)
+
+    # Address handling
+
+    def get_address_host(self, loc):
+        return parse_host_port(loc)[0]
+
+    def get_address_host_port(self, loc):
+        return parse_host_port(loc)
+
+    def resolve_address(self, loc):
+        host, port = parse_host_port(loc)
+        return unparse_host_port(ensure_ip(host), port)
+
+    def get_local_address_for(self, loc):
+        host, port = parse_host_port(loc)
+        host = ensure_ip(host)
+        if ':' in host:
+            local_host = get_ipv6(host)
+        else:
+            local_host = get_ip(host)
+        return unparse_host_port(local_host, None)
+
+
+backends['tcp'] = TCPBackend()
