@@ -3,6 +3,7 @@ from __future__ import print_function, division, absolute_import
 from functools import partial
 import sys
 from time import sleep
+from threading import Lock
 import unittest
 
 from tornado.ioloop import IOLoop
@@ -12,12 +13,14 @@ import pytest
 from distributed import Client, Worker, Nanny
 from distributed.deploy.local import LocalCluster
 from distributed.metrics import time
-from distributed.utils_test import (inc, loop, raises, gen_test,
+from distributed.utils_test import (inc, loop, raises, gen_test, pristine_loop,
         assert_can_connect_locally_4, assert_can_connect_from_everywhere_4_6)
 from distributed.utils import ignoring, sync
+from distributed.worker import TOTAL_MEMORY, _ncores
 
 from distributed.deploy.utils_test import ClusterTest
 
+
 def test_simple(loop):
     with LocalCluster(4, scheduler_port=0, nanny=False, silence_logs=False,
                       diagnostics_port=None, loop=loop) as c:
@@ -36,6 +39,7 @@ def test_procs(loop):
         assert all(isinstance(w, Worker) for w in c.workers)
         with Client(c.scheduler.address, loop=loop) as e:
             assert all(w.ncores == 3 for w in c.workers)
+            assert all(isinstance(w, Worker) for w in c.workers)
         repr(c)
 
     with LocalCluster(2, scheduler_port=0, nanny=True, threads_per_worker=3,
@@ -45,11 +49,56 @@ def test_procs(loop):
         with Client(c.scheduler.address, loop=loop) as e:
             assert all(v == 3 for v in e.ncores().values())
 
-            c.start_worker(nanny=False)
-            assert isinstance(c.workers[-1], Worker)
+            c.start_worker()
+            assert all(isinstance(w, Nanny) for w in c.workers)
         repr(c)
 
 
+def test_move_unserializable_data():
+    """
+    Test that unserializable data is still fine to transfer over inproc
+    transports.
+    """
+    with LocalCluster(nanny=False, silence_logs=False,
+                      diagnostics_port=None) as cluster:
+        assert cluster.scheduler_address.startswith('inproc://')
+        assert cluster.workers[0].address.startswith('inproc://')
+        with Client(cluster) as client:
+            lock = Lock()
+            [x] = client.scatter([lock])
+            y = client.submit(lambda x: x, x)
+            assert y.result() is lock
+
+
+def test_transports():
+    """
+    Test the transport chosen by LocalCluster depending on arguments.
+    """
+    with LocalCluster(1, nanny=False, silence_logs=False,
+            diagnostics_port=None) as c:
+        assert c.scheduler_address.startswith('inproc://')
+        assert c.workers[0].address.startswith('inproc://')
+        with Client(c.scheduler.address) as e:
+            assert e.submit(inc, 4).result() == 5
+
+    # Have nannies => need TCP
+    with LocalCluster(1, nanny=True, silence_logs=False,
+                      diagnostics_port=None) as c:
+        assert c.scheduler_address.startswith('tcp://')
+        assert c.workers[0].address.startswith('tcp://')
+        with Client(c.scheduler.address) as e:
+            assert e.submit(inc, 4).result() == 5
+
+    # Scheduler port specified => need TCP
+    with LocalCluster(1, nanny=False, scheduler_port=8786, silence_logs=False,
+                      diagnostics_port=None) as c:
+
+        assert c.scheduler_address == 'tcp://127.0.0.1:8786'
+        assert c.workers[0].address.startswith('tcp://')
+        with Client(c.scheduler.address) as e:
+            assert e.submit(inc, 4).result() == 5
+
+
 @pytest.mark.skipif('sys.version_info[0] == 2', reason='')
 class LocalTest(ClusterTest, unittest.TestCase):
     Cluster = partial(LocalCluster, silence_logs=False, diagnostics_port=None)
@@ -148,6 +197,7 @@ def test_http(loop):
 
 
 def test_bokeh(loop):
+    pytest.importorskip('bokeh')
     from distributed.http import HTTPScheduler
     import requests
     with LocalCluster(scheduler_port=0, silence_logs=False, loop=loop,
@@ -170,6 +220,7 @@ def test_bokeh(loop):
 
 
 def test_start_diagnostics(loop):
+    pytest.importorskip('bokeh')
     from distributed.http import HTTPScheduler
     import requests
     with LocalCluster(scheduler_port=0, silence_logs=False, loop=loop,
@@ -236,3 +287,18 @@ def test_remote_access(loop):
     with LocalCluster(scheduler_port=0, silence_logs=False,
                       diagnostics_port=None, ip='', loop=loop) as c:
         sync(loop, assert_can_connect_from_everywhere_4_6, c.scheduler.port)
+
+
+def test_memory(loop):
+    with LocalCluster(scheduler_port=0, nanny=False, silence_logs=False,
+                      diagnostics_port=None, loop=loop) as cluster:
+        assert sum(w.memory_limit for w in cluster.workers) < TOTAL_MEMORY * 0.8
+
+
+def test_memory_nanny(loop):
+    with LocalCluster(scheduler_port=0, nanny=True, silence_logs=False,
+                      diagnostics_port=None, loop=loop) as cluster:
+        with Client(cluster.scheduler_address, loop=loop) as c:
+            info = c.scheduler_info()
+            assert (sum(w['memory_limit'] for w in info['workers'].values())
+                    < TOTAL_MEMORY * 0.8)
