@@ -175,17 +175,23 @@ def color_of_message(msg):
 
 
 colors = {'transfer': 'red',
-          'disk': 'orange',
+          'disk-write': 'orange',
+          'disk-read': 'orange',
+          'deserialize': 'gray',
           'compute': color_of_message}
 
 
 alphas = {'transfer': 0.4,
           'compute': 1,
-          'disk': 0.4}
+          'deserialize': 0.4,
+          'disk-write': 0.4,
+          'disk-read': 0.4}
 
 
 prefix = {'transfer': 'transfer-',
-          'disk': 'load-',
+          'disk-write': 'disk-write-',
+          'disk-read': 'disk-read-',
+          'deserialize': 'deserialize-',
           'compute': ''}
 
 
