@@ -2,8 +2,11 @@ from __future__ import print_function, division, absolute_import
 
 from datetime import datetime
 import os
+import random
 import sys
 
+import numpy as np
+
 import pytest
 from toolz import valmap
 from tornado.tcpclient import TCPClient
@@ -14,7 +17,7 @@ from distributed.core import connect, CommClosedError
 from distributed.metrics import time
 from distributed.protocol.pickle import dumps, loads
 from distributed.utils import ignoring
-from distributed.utils_test import gen_cluster
+from distributed.utils_test import gen_cluster, gen_test
 from distributed.nanny import isalive
 
 
@@ -151,3 +154,27 @@ def test_close_on_disconnect(s, w):
     while w.status != 'closed':
         yield gen.sleep(0.01)
         assert time() < start + 9
+
+
+@gen_test()
+def test_nanny_death_timeout():
+    w = Nanny('127.0.0.1', 38848, death_timeout=1)
+    yield w._start()
+
+    yield gen.sleep(3)
+    assert w.status == 'closed'
+
+
+@gen_cluster(client=True, Worker=Nanny)
+def test_random_seed(c, s, a, b):
+    @gen.coroutine
+    def check_func(func):
+        x = c.submit(func, 0, 2**31, pure=False, workers=a.worker_address)
+        y = c.submit(func, 0, 2**31, pure=False, workers=b.worker_address)
+        assert x.key != y.key
+        x = yield x._result()
+        y = yield y._result()
+        assert x != y
+
+    yield check_func(lambda a, b: random.randint(a, b))
+    yield check_func(lambda a, b: np.random.randint(a, b))
