@@ -19,11 +19,9 @@ def test_gather_from_workers_permissive(s, a, b):
     while not a.batched_stream:
         yield gen.sleep(0.01)
     a.update_data(data={'x': 1})
-    with pytest.raises(KeyError):
-        yield gather_from_workers({'x': [a.address], 'y': [b.address]})
 
     data, missing, bad_workers = yield gather_from_workers(
-            {'x': [a.address], 'y': [b.address]}, permissive=True)
+            {'x': [a.address], 'y': [b.address]})
 
     assert data == {'x': 1}
     assert list(missing) == ['y']
